unit TestSwitch;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, Math, FPImage, FPCanvas, LResources, Forms, Controls,
  Graphics, Dialogs, ExtCtrls, IntfGraphics, LCLProc, LCLType, LCLIntf,
  GraphType, lazcanvas;

type
  TClickEvent = procedure(Sender: TObject) of object;

type
  TDirection = (goRight,goLeft);

type

  { TTestSwitch }

  TTestSwitch = class(TCustomControl)
  private
   FFinalBgrdColor: TColor;
   FImages     : Array[0..2] of TCustomBitmap;
   FInitialFileImage : TCustomBitmap;
   FFinalFileImage : TCustomBitmap;
   FInitialBgrdColor: TColor;
   FInitialRollimg : TLazIntfImage;
   FFinalRollImg   : TLazIntfImage;
   FMargin     : Integer;
   FOnClick: TClickEvent;
   FRoll: boolean;
   FRollImgSize: Integer;
   FRollPos    : Integer;
   FAngel      : Double;
   FPortion    : Double;
   FOldWidth   : integer;
   FOldHeight  : integer;
   FSpeed: integer;
   FTimer      : TTimer;
   FRotation   : Integer;
   FDirection  : TDirection;
   procedure CalculateRollImage;
   procedure FTimerTimer(Sender: TObject);
   procedure SetFinalBgrdColor(AValue: TColor);
   procedure SetInitialBgrdColor(AValue: TColor);
   procedure SetSpeed(AValue: integer);
  protected
   procedure BoundsChanged;override;
  public
   constructor Create(AOwner: TComponent); override;
   destructor  Destroy; override;
   procedure LoadImagesfromFile(InitialFilename,FinalFilename: string);
   procedure   Paint; override;
   procedure MouseMove({%H-}Shift: TShiftState; X, Y: Integer);override;
   procedure MouseDown({%H-}Button: TMouseButton;{%H-}Shift: TShiftState; X, Y: Integer);override;
   procedure MouseUp({%H-}Button: TMouseButton; {%H-}Shift: TShiftState; {%H-}X, {%H-}Y: Integer);override;

   property Direction : TDirection read FDirection write FDirection;
  published
   property InitialBgrdColor : TColor read FInitialBgrdColor write SetInitialBgrdColor default $000000C8;
   property FinalBgrdColor   : TColor read FFinalBgrdColor write SetFinalBgrdColor default $0000C800;
   property Speed : integer read FSpeed write SetSpeed default 10;
   property Roll : boolean read FRoll write FRoll default true;

   property OnClick : TClickEvent read FOnClick     write FOnClick;
  end;

procedure Register;

implementation

procedure Register;
begin
  {$I testswitch_icon.lrs}
  RegisterComponents('Misc',[TTestSwitch]);
  {$R images.res}
end;


function RotatePoint(const APoint: TPoint; AAngle: Double): TPoint;
var
  sa, ca: Double;
begin
  SinCos(AAngle, sa, ca);
  Result.X := Round( ca * APoint.X + sa * APoint.Y);
  Result.Y := Round(-sa * APoint.X + ca * APoint.Y);
end;

procedure RotateImage(Img: TFPCustomImage; Angle: Double);
var
  Buffer: TFPCustomImage;
  x, y: Integer;
  C, P: TPoint;
begin
  Buffer := TFPMemoryImage.Create(Img.Width, Img.Height);
  C := Point(Img.Width div 2, Img.Height div 2);
  for y := 0 to Buffer.Height-1 do
    for x := 0 to Buffer.Width-1 do
    begin
      P := RotatePoint(Point(x, y) - C, Angle) + C;

      if (P.X >= 0) and (P.Y >= 0) and (P.X < Img.Width) and (P.Y < Img.Height) then
        Buffer.Colors[x, y] := Img.Colors[P.X, P.Y]
      else
        Buffer.Colors[x, y] := colTransparent;
    end;
  for y := 0 to Img.Height-1 do
   for x := 0 to Img.Width-1 do
      Img.Colors[x, y] := Buffer[x, y];
  Buffer.Free;
end;

procedure BlendImages(Img1, Img2: TFPCustomImage; AFactor: Double);
var
  x, y: Integer;
  r,g,b,a: Word;
  f1, f2: Double;
begin
  f1 := 1.0 - AFactor;
  f2 := AFactor;
  for y := 0 to Img1.Height-1 do
    for x := 0 to Img1.Width-1 do
    begin
      r := round(f1 * Img1.Colors[x, y].Red   + f2 * Img2.Colors[x, y].Red);
      g := round(f1 * Img1.Colors[x, y].Green + f2 * Img2.Colors[x, y].Green);
      b := round(f1 * Img1.Colors[x, y].Blue  + f2 * Img2.Colors[x, y].Blue);
      a := round(f1 * Img1.Colors[x, y].Alpha + f2 * Img2.Colors[x, y].Alpha);
      Img1.Colors[x, y] := FPColor(r, g, b, a);
    end;
end;

procedure ChangeColor(Img: TLazIntfImage;aColor : TColor);
var Image1                 : TCustomBitmap;
    Image2                 : TLazIntfImage;
    valR,valG,valB,valA    : byte;
    valRNew,valGNew,valBNew: byte;
    valtemp1,valtemp2      : byte;
    x,y                    : integer;
    P                      : TPoint;
begin
 Image1 := TPortableNetworkGraphic.Create;
 Image1.SetSize(Img.Width,Img.Height);
 Image2:= Image1.CreateIntfImage;
 Image2.Assign(Img);
 {$IFDEF Windows}
  valtemp1:=0;
  valtemp2:=200;
  valRNew:=GetRValue(aColor);
  valGNew:=getGvalue(aColor);
  valBNew:=getBvalue(aColor);
 {$ENDIF}
 {$IFDEF LINUX}
  valRNew:=GetBValue(aColor);
  valGNew:=getGvalue(aColor);
  valBNew:=getRvalue(aColor);
  valtemp1:=200;
  valtemp2:=0;
 {$ENDIF}
 try

   for y := 0 to  Img.height - 1 do
    begin
     for x := 0 to Img.width - 1 do
      begin
       P.X:=x;P.Y:=y;
       valR:= PRGBQUAD(Img.GetDataLineStart(P.Y))[P.X].rgbRed;
       valG:= PRGBQuad(Img.GetDataLineStart(P.Y))[P.X].rgbGreen;
       valB:= PRGBQuad(Img.GetDataLineStart(P.Y))[P.X].rgbBlue;
       valA:= PRGBQuad(Img.GetDataLineStart(P.Y))[P.X].rgbReserved;
       //if valA <> 0 then
       if valR = valtemp2 then
        if valG = 0 then
         if valB = valtemp1 then
          if ValA = 255 then
           begin
            PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbRed    := valRNew;
            PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbGreen  := valGNew;
            PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbBlue   := valBNew;
            PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbReserved:=255;
           end;
       if (ValA <> 255) and (valA <> 0) then
           begin
            PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbRed    := valRNew;
            PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbGreen  := valGNew;
            PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbBlue   := valBNew;
            PRGBQuad(Image2.GetDataLineStart(P.Y))[P.X].rgbReserved:=valA;
           end;

      end;//for x
    end;//for y
    Img.Assign(Image2);

 finally
  Image2.Free;
  Image1.Free;
 end;
end;

Procedure StretchDrawImgToImg(SourceImg, DestImg: TLazIntfImage; DestWidth, DestHeight: integer);
Var
  DestIntfImage: TLazIntfImage;
  DestCanvas: TLazCanvas;
Begin
  DestIntfImage := TLazIntfImage.Create(0, 0);
  DestIntfImage.Assign(DestImg);
  DestCanvas := TLazCanvas.Create(DestIntfImage);
  DestCanvas.Interpolation := TMitchelInterpolation.Create;//TFPSharpInterpolation.Create;
  DestCanvas.StretchDraw(0, 0, DestWidth, DestHeight, SourceImg);
  DestImg.Assign(DestIntfImage);
  DestCanvas.Interpolation.Free;
  DestCanvas.Free;
  DestIntfImage.Free;
End;

function System_ToRGB(clSys:TColor):TColor;
  var FPCol :  TFPColor;
  begin
   FPCol:=TColorToFPColor(ColorToRGB(clSys));
   result :=FPColorToTColor(FPCol);
  end;
//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
{ TTestSwitch }
procedure TTestSwitch.BoundsChanged;
var Factor : double;
begin
  inherited BoundsChanged;

  if width <> FOldWidth then
   begin
    Factor := Width / 60;
    Height := round(Factor * 26);
    FOldWidth := Width;
   end;
  if Height <> FOldHeight then
   begin
    Factor := Height / 26;
    Width := round(Factor * 60);
    FOldHeight := Height;
   end;
  Invalidate;
end;

constructor TTestSwitch.Create(AOwner: TComponent);
var lv : integer;
begin
  inherited Create(AOwner);
  Width      := 60;
  FOldWidth  := 60;
  Height     := 26;
  FOldHeight := 26;
  FMargin    := 3;
  FRollPos   := 0;
  FRotation  := 30;
  FAngel     := 0;
  FPortion   := 0;
  FDirection := goLeft;
  FRoll      := true;
  FInitialBgrdColor := rgb(200,0,0);
  FFinalBgrdColor   := rgb(0,200,0);

  FTimer            := TTimer.Create(nil);
  FSpeed            := 10;
  FTimer.Interval   := FSpeed;
  FTimer.Enabled    := false;
  FTimer.OnTimer    :=@FTimerTimer;

  for lv := 0 to High(FImages) do
    FImages[lv] := TPortableNetworkGraphic.Create;
  FImages[0].LoadFromResourceName(HInstance,'backbround_red180_78');
  FImages[1].LoadFromResourceName(HInstance,'off');
  FImages[2].LoadFromResourceName(HInstance,'ok');

end;

destructor TTestSwitch.Destroy;
var lv : integer;
begin
 for lv := 0 to High(FImages) do
    FImages[lv].Free;
 if assigned (FInitialFileImage) then FInitialFileImage.Free;
 if assigned (FFinalFileImage) then FFinalFileImage.Free;
 FTimer.Free;
 inherited Destroy;
end;

procedure TTestSwitch.LoadImagesfromFile(InitialFilename, FinalFilename: string);
begin
 if not fileexists(InitialFilename) or not fileexists(FinalFilename) then
  begin
   if assigned (FInitialFileImage) then FreeAndNil(FInitialFileImage);
   if assigned (FFinalFileImage) then FreeAndNil(FFinalFileImage);
   Invalidate;
   exit;
  end;
 if assigned (FInitialFileImage) then FreeAndNil(FInitialFileImage);
 if not assigned(FInitialFileImage) then
  FInitialFileImage := TPortableNetworkGraphic.Create;
 FInitialFileImage.LoadFromFile(InitialFilename);
 if assigned (FFinalFileImage) then FreeAndNil(FFinalFileImage);
 if not assigned(FFinalFileImage) then
  FFinalFileImage := TPortableNetworkGraphic.Create;
 FFinalFileImage.LoadFromFile(FinalFilename);

 if (FInitialFileImage.Width <> FFinalFileImage.Width) or (FInitialFileImage.Height <> FFinalFileImage.Height) then
  begin
   FreeAndNil(FInitialFileImage);
   FreeAndNil(FFinalFileImage);
   showmessage('The size of the images must be the same!');
  end;
 Invalidate;
end;


procedure TTestSwitch.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove(Shift, X, Y);
end;

procedure TTestSwitch.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TTestSwitch.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if not FTimer.Enabled then
   begin
    //if FDirection = goRight then FDirection := goLeft else FDirection := goRight;
    FDirection := TDirection((ord(FDirection) + 1) mod 2);
    FTimer.Enabled:= true;
    if Assigned(OnClick) then OnClick(self);
   end;
end;

procedure TTestSwitch.CalculateRollImage;
var Factor : double;
begin
 Factor       := (2 * 100) / 26;
 FMargin      := round((Height / 100) * Factor);

 FRollImgSize := Height - (2 * FMargin);
end;

procedure TTestSwitch.FTimerTimer(Sender: TObject);
var l,l1 : integer;
    f    : double;
begin
  l  := width - Height;
  l1 := round(l / (360/FRotation));
  f  := 1 / (360/FRotation);
  if FDirection = goRight then
   begin
    FPortion := FPortion + f;
    FRollPos := FRollPos + l1;
    FAngel :=FAngel+FRotation;
    if FAngel >= 360 then
     begin
      FPortion := 1;
      FTimer.Enabled:= false;
      FRollPos := width - (FRollImgSize + (2*FMargin));
     end;
   end;
   if FDirection = goLeft then
   begin
    FPortion := FPortion - f;
    FRollPos := FRollPos - l1;
    FAngel := FAngel - FRotation;
    f  := 1 / (360/FRotation);
    if FAngel <= 0 then
     begin
      FPortion := 0;
      FTimer.Enabled:= false;
      FRollPos := 0 ;
     end;
   end;
   Invalidate;
 end;

procedure TTestSwitch.SetFinalBgrdColor(AValue: TColor);
begin
  if FFinalBgrdColor=AValue then Exit;
  FFinalBgrdColor:=System_ToRGB(AValue);
  Invalidate;
end;

procedure TTestSwitch.SetInitialBgrdColor(AValue: TColor);
begin
  if FInitialBgrdColor=AValue then Exit;
  FInitialBgrdColor:=System_ToRGB(AValue);
  Invalidate;
end;

procedure TTestSwitch.SetSpeed(AValue: integer);
begin
  if FSpeed=AValue then Exit;
  FSpeed:=AValue;
  FTimer.Interval   := FSpeed;
end;

procedure TTestSwitch.Paint;
var RollBmp, BackBmp  : TBitmap;
    BackImg1,BackImg2,BackImg3 : TLazIntfImage;
begin
 if width < 10 then width := 10;

 BackImg1 := FImages[0].CreateIntfImage;
 ChangeColor(BackImg1,FInitialBgrdColor);

 BackImg2 := FImages[0].CreateIntfImage;
 ChangeColor(BackImg2,FFinalBgrdColor);

 BlendImages(BackImg1,BackImg2,FPortion);

 BackImg3 := TLazIntfImage.Create(0,0, [riqfRGB, riqfAlpha]);
 BackImg3.SetSize(BackImg1.Width,BackImg1.Height);
 StretchDrawImgToImg(BackImg1,BackImg3,width,height);


 BackBmp  := TBitmap.Create;
 BackBmp.PixelFormat:= pf32Bit;
 BackBmp.Assign(BackImg3);
 Canvas.Draw(0,0,BackBmp);
 //Canvas.StretchDraw(Rect(0,0,width,height),BackBmp);

 Backbmp.Free;

 BackImg2.Free;
 BackImg1.Free;
 BackImg3.Free;

 CalculateRollImage;

 if assigned (FInitialFileImage) then FInitialRollImg := FInitialFileImage.CreateIntfImage
 else FInitialRollImg := FImages[1].CreateIntfImage;

 if assigned (FFinalFileImage) then FFinalRollImg := FFinalFileImage.CreateIntfImage
 else FFinalRollImg   := FImages[2].CreateIntfImage;

 RollBmp     := TBitmap.Create;
 try
  BlendImages(FInitialRollImg,FFinalRollImg,FPortion);
  if FRoll then
   RotateImage(FInitialRollImg,degtorad(FAngel));
  RollBmp.Pixelformat := pf32Bit;

  BackImg1 := TLazIntfImage.Create(0,0, [riqfRGB, riqfAlpha]);
  BackImg1.SetSize(FRollImgSize,FRollImgSize);
  StretchDrawImgToImg(FInitialRollImg,BackImg1,FRollImgSize,FRollImgSize);
  RollBmp.Assign(BackImg1);

  if (FDirection = goRight) and not FTimer.Enabled then
   FRollPos := width - (FRollImgSize + (2*FMargin));

  Canvas.Draw(FMargin+FRollPos,FMargin,RollBmp);


 finally
  RollBmp.Free;
  FInitialRollImg.Free;
  FFinalRollImg.Free;
  BackImg1.Free;
 end;



end;


end.


